@@ -3,7 +3,7 @@ Json tools to serialized and deserialze data objects
 """
 
 import json
-from typing import Type, Generic, Optional
+from typing import Type, Generic, Optional, Union
 
 from dataclasses_jsonschema import ValidationError
 
@@ -15,7 +15,7 @@ class Json(Generic[EventPayloadType]):
     Json convenience ser/deser functions for @dataobject decorated object (@see DataObject)
     """
     @staticmethod
-    def from_json(json_str: str,
+    def from_json(json_str: Union[str, bytes],
                   datatype: Type[EventPayloadType],
                   key: str = 'value') -> EventPayloadType:
         """
@@ -62,3 +62,11 @@ class Json(Generic[EventPayloadType]):
             return payload.to_json(validate=payload.__data_object__['validate'])  # type: ignore
         except (ValidationError, AttributeError) as e:
             raise ValueError(f"Cannot convert to JSON: type={type(payload)} validation_error={str(e)}") from e
+
+    @staticmethod
+    def parse_form_field(field_data: Union[str, dict], datatype: Type[EventPayloadType],
+                         key: str = 'value') -> EventPayloadType:
+        """Helper to parse dataobjects from form-fields where encoding type is not correctly set to json"""
+        if isinstance(field_data, str):
+            return Json.from_json(field_data, datatype, key)
+        return datatype.from_dict(field_data)  # type: ignore
